#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "iir.h"

#define FUNJOIN(p1, p2, p3) FUNJOIN2(p1, p2, p3)
#define FUNJOIN2(p1, p2, p3) p1 ## p2 ## p3


//#define ITERATIONS 10000
#define ITERATIONS 100

int main (int argc, char *argv[])
{
  int acoeffs;
  int size = 1048576;
  //int size = 64 * 1024;
  int i;
  clock_t t1, t2, t3;
  clock_t pms = CLOCKS_PER_SEC / 1000;

  float *msource    = (float *) malloc (size * sizeof (float) + 100);
  float *source = msource + 50;
  float *destplain = (float *) malloc (size * sizeof (float));
  float *mdestvec   = (float *) malloc (size * sizeof (float) + 100);
  float *destvec = mdestvec + 50;
  double error, diff, verror;

  if (argc < 2 || argc > 2)
  {  fputs ("usage: test_iir a-coeff-count\n", stderr);
     exit (1);
  }

  acoeffs = atoi (argv[1]);

  while ((int) source % 16) source ++;
  while ((int) destvec % 16) destvec ++;

  for (i = 0; i < size; i ++)
      source[i] = i * 0.1;
  for (i = 0; i < 100; i ++)
  {
      msource[i] = 0.0;
      msource[size + 99 - i] = 0.0;
  }

  t1 = clock ();

  for (i = 0; i < ITERATIONS; i ++)
    switch (acoeffs)
    { case 4: iir_4_9_plain (source, destplain, size); break;
      case 5: iir_5_8_plain (source, destplain, size); break;
      case 6: iir_6_7_plain (source, destplain, size); break;
      case 7: iir_7_6_plain (source, destplain, size); break;
      case 8: iir_8_5_plain (source, destplain, size); break;
      case 9: iir_9_4_plain (source, destplain, size); break;
      default:
        fputs ("4 <= a-coeff-count <= 9\n", stderr);
        exit (1);
    }

  t2 = clock ();

  for (i = 0; i < ITERATIONS; i ++)
    switch (acoeffs)
    { case 4: iir_4_9_vec (source, destvec, size); break;
      case 5: iir_5_8_vec (source, destvec, size); break;
      case 6: iir_6_7_vec (source, destvec, size); break;
      case 7: iir_7_6_vec (source, destvec, size); break;
      case 8: iir_8_5_vec (source, destvec, size); break;
      case 9: iir_9_4_vec (source, destvec, size); break;
    }

  t3 = clock ();

  //for (i=0;i<400;i++) printf ("%f %f\n", destplain[i], destvec[i]);

  error = 0.0;
  for (i = 0; i < size; i ++)
  {
      diff = destplain[i] - destvec[i];
      error += diff * diff;
  }
  error /= (double) size;

  printf ( " plain time: %d ms\n vector time: %d ms\n speedup: %f\n error: %f\n"
         , (t2 - t1) / pms
         , (t3 - t2) / pms
         , (double) (t2 - t1) / (double) (t3 - t2)
         , error
         );

  free (msource);
  free (destplain);
  free (mdestvec);
}
